(* Top-level of the MicroC compiler: scan & parse the input,
   check the resulting AST, generate LLVM IR, and dump the module *)

type action = Ast | Sast| LLVM_IR | Compile

let _ =
  let action = ref Compile in
  let input = ref "" in
  let set_action a () = action := a in
  let speclist = [
    ("-a", Arg.Unit (set_action Ast), "Print the AST");
    ("-s", Arg.Unit (set_action Sast), "Print the SAST");
    ("-l", Arg.Unit (set_action LLVM_IR), "Print the generated LLVM IR");
    ("-c", Arg.Unit (set_action Compile),
      "Check and print the generated LLVM IR (default)");
  ] in
  let usage_msg = "usage: ./scolang.native [-a|-l] [file]" in
  Arg.parse speclist (fun s -> input := s) usage_msg;
  let channel = if !input = "" then
    stdin
  else
    open_in !input
  in
  let lexbuf = Lexing.from_channel channel in (*reading in the input*)
  let ast = Parser.start_state (*parser takes the tokens, pattern matches builds an abstract syntax tree*) Scanner.token lexbuf (*scanner takes lexbuf and adds tokens to it *)in
  let sast = Semant.check ast in (*Semantically checks it, adds types where missing and creates SAST*)
  match !action with
    Ast -> print_string (Ast.string_of_program [] ast)
  | Sast -> print_string (Sast.string_of_sprogram sast)
  | LLVM_IR -> print_string (Llvm.string_of_llmodule (Codegen.translate sast))
  | Compile -> let m = Codegen.translate sast (*codegen takes sast, matches llvm api's to our code and generates some form of IR *) in
    Llvm_analysis.assert_valid_module m; (*LLVM does syntax checking on IR*)
    print_string (Llvm.string_of_llmodule m) (*Prints out IR*)
